==> o365-3.0.1.tar.gz: o365/modules/o365_sso_user/src/SsoUserSync.php <==
<?php

namespace Drupal\o365_sso_user;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\o365\GraphService;
use Drupal\o365_profile\O365ProfileGetDataService;
use Drupal\user\UserInterface;
use Mimey\MimeTypes;

/**
 * User sync service that syncs data from Graph API to entities.
 */
class SsoUserSync {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The modules config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * The o365 Graph Service.
   *
   * @var \Drupal\o365\GraphService
   */
  protected $graphService;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The profile data service.
   *
   * @var \Drupal\o365_profile\O365ProfileGetDataService
   */
  protected $profileGetDataService;

  /**
   * Constructs a SsoUserSync object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\o365\GraphService $graphService
   *   The Graph API service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\o365_profile\O365ProfileGetDataService $o365ProfileGetDataService
   *   The profile data service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, GraphService $graphService, EntityTypeManagerInterface $entityTypeManager, O365ProfileGetDataService $o365ProfileGetDataService) {
    $this->configFactory = $config_factory;
    $this->graphService = $graphService;
    $this->entityTypeManager = $entityTypeManager;
    $this->profileGetDataService = $o365ProfileGetDataService;
    $this->config = $config_factory->get('o365_sso_user.settings');
  }

  /**
   * Sync the user data with data from the graph API.
   */
  public function synUserData($user) {
    // Only do this when we want to use graph data.
    if (!empty($this->config->get('use_graph_data'))) {
      // Get user data.
      $userData = $this->profileGetDataService->getProfileData(NULL, '648x648');
      if ($userData) {
        $entityAltered = FALSE;

        // Sync the account name with the display name.
        if ($userData['userData']['displayName'] !== $user->getAccountName()) {
          $user->setUsername($userData['userData']['displayName']);
          $entityAltered = TRUE;
        }

        if (!empty($userData)) {
          if ($this->config->get('use_profile_module')) {
            $this->syncUserProfileData($user, $userData);
          }
          else {
            $this->synUserUserData($user, $userData, $entityAltered);
          }
        }

        // Save the user when it has changed.
        if ($entityAltered) {
          $user->save();
        }
      }
    }
  }

  /**
   * Sync a standard drupal user.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user object.
   * @param array $userData
   *   The user data from the Graph API.
   * @param bool $entityAltered
   *   If the entity is altered or not. Used later on.
   */
  private function synUserUserData(UserInterface $user, array $userData, &$entityAltered) {
    $this->mapDataToFields($user, $userData['userData'], $entityAltered);

    // Sync the user picture.
    $this->syncUserPicture($user, $entityAltered, $userData);
  }

  /**
   * Sync a Drupal user using the profile module.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user object.
   * @param array $userData
   *   The user data from the Graph API.
   */
  private function syncUserProfileData(UserInterface $user, array $userData) {
    // First get the profile for this user.
    $storage = $this->entityTypeManager->getStorage('profile');

    $profileBundle = ($this->config->get('profile_bundle')) ?: 'profile';
    $profiles = $storage->loadMultipleByUser($user, $profileBundle);
    if (!empty($profiles)) {
      $entityAltered = FALSE;
      /** @var \Drupal\profile\Entity\Profile $profile */
      $profile = reset($profiles);
      $this->mapDataToFields($profile, $userData['userData'], $entityAltered);

      // Sync the user picture.
      $this->syncUserPicture($profile, $entityAltered, $userData);

      if ($entityAltered) {
        $profile->save();
      }
    }
  }

  /**
   * Sync the user picture if needed and wanted.
   *
   * @param \Drupal\user\UserInterface|\Drupal\profile\Entity\Profile $entity
   *   The user or profile entity.
   * @param bool $entityAltered
   *   If the entity is altered or not. Used later on.
   * @param array $userData
   *   The user data array containing the image.
   */
  private function syncUserPicture(&$entity, &$entityAltered, array $userData) {
    if (!empty($this->config->get('sync_profile_image'))) {
      $entityField = $this->config->get('profile_image_mapping');
      if ($entity->hasField($entityField)) {
        if ($userData && $userData['imageBase64']) {
          $data = base64_decode($userData['imageBase64']);

          // Setup the image file uri.
          $fieldDefinition = $entity->getFieldDefinition($entityField);
          if ($fieldDefinition) {
            $fieldDefinitionSettings = $fieldDefinition->getSettings();
            $uri = $fieldDefinitionSettings['uri_scheme'] . '://';
            $mimes = new MimeTypes();
            $imageName = $entity->id() . '.' . $mimes->getExtension($userData['imageMeta']['@odata.mediaContentType']);

            $imageFile = file_save_data($data, $uri . $imageName, FileSystemInterface::EXISTS_REPLACE);

            // Save the file to the entity .
            if ($imageFile) {
              $entity->set($entityField, $imageFile->id());
              $entityAltered = TRUE;
            }
          }
        }
      }
    }
  }

  /**
   * Map the user data to field in a entity.
   *
   * @param \Drupal\user\UserInterface|\Drupal\profile\Entity\Profile $entity
   *   The user or profile entity.
   * @param array $userData
   *   The user data.
   * @param bool $entityAltered
   *   If the entity is altered or not. Used later on.
   */
  private function mapDataToFields(&$entity, array $userData, &$entityAltered = FALSE) {
    $mapping = $this->getFieldMapping();

    if (!empty($mapping)) {
      // Loop through the mappings.
      foreach ($mapping as $userDataField => $entityField) {
        // Check if the fields exist in both the data and the user.
        if (isset($userData[$userDataField]) && $entity->hasField($entityField)) {
          $entity->set($entityField, $userData[$userDataField]);
          $entityAltered = TRUE;
        }
      }
    }
  }

  /**
   * Get the field mappings from the settings.
   *
   * @return array
   *   The mapping as graph_api_field => drupal_field.
   */
  private function getFieldMapping() {
    $mapping = [];
    $data = $this->config->get('graph_data_mapping');
    $lines = preg_split('/\r\n|\r|\n/', $data);

    if (!empty($lines)) {
      foreach ($lines as $line) {
        $field = explode('|', $line);
        $mapping[trim($field[0])] = trim($field[1]);
      }
    }

    return $mapping;
  }

}

==> o365-3.0.15.tar.gz: o365/modules/o365_sso_user/src/SsoUserSync.php <==
<?php

namespace Drupal\o365_sso_user;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\o365\GraphService;
use Drupal\o365_profile\O365ProfileGetDataService;
use Drupal\user\UserInterface;
use Mimey\MimeTypes;

/**
 * User sync service that syncs data from Graph API to entities.
 */
class SsoUserSync {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The modules config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * The o365 Graph Service.
   *
   * @var \Drupal\o365\GraphService
   */
  protected $graphService;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The profile data service.
   *
   * @var \Drupal\o365_profile\O365ProfileGetDataService
   */
  protected $profileGetDataService;

  /**
   * Constructs a SsoUserSync object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\o365\GraphService $graphService
   *   The Graph API service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\o365_profile\O365ProfileGetDataService $o365ProfileGetDataService
   *   The profile data service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, GraphService $graphService, EntityTypeManagerInterface $entityTypeManager, O365ProfileGetDataService $o365ProfileGetDataService) {
    $this->configFactory = $config_factory;
    $this->graphService = $graphService;
    $this->entityTypeManager = $entityTypeManager;
    $this->profileGetDataService = $o365ProfileGetDataService;
    $this->config = $config_factory->get('o365_sso_user.settings');
  }

  /**
   * Sync the user data with data from the graph API.
   */
  public function syncUserData($user) {
    // Only do this when we want to use graph data.
    if (!empty($this->config->get('use_graph_data'))) {
      // Get user data.
      $userData = $this->profileGetDataService->getProfileData(NULL, '648x648');
      if ($userData) {
        $entityAltered = FALSE;

        // Sync the account name with the display name.
        if ($userData['userData']['displayName'] !== $user->getAccountName()) {
          $user->setUsername($userData['userData']['displayName']);
          $entityAltered = TRUE;
        }

        if (!empty($userData)) {
          if ($this->config->get('use_profile_module')) {
            $this->syncUserProfileData($user, $userData);
          }
          else {
            $this->synUserUserData($user, $userData, $entityAltered);
          }
        }

        // Save the user when it has changed.
        if ($entityAltered) {
          $user->save();
        }
      }
    }
  }

  /**
   * Sync a standard drupal user.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user object.
   * @param array $userData
   *   The user data from the Graph API.
   * @param bool $entityAltered
   *   If the entity is altered or not. Used later on.
   */
  private function synUserUserData(UserInterface $user, array $userData, &$entityAltered) {
    $this->mapDataToFields($user, $userData['userData'], $entityAltered);

    // Sync the user picture.
    $this->syncUserPicture($user, $entityAltered, $userData);
  }

  /**
   * Sync a Drupal user using the profile module.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user object.
   * @param array $userData
   *   The user data from the Graph API.
   */
  private function syncUserProfileData(UserInterface $user, array $userData) {
    // First get the profile for this user.
    $storage = $this->entityTypeManager->getStorage('profile');

    $profileBundle = ($this->config->get('profile_bundle')) ?: 'profile';
    $profiles = $storage->loadMultipleByUser($user, $profileBundle);
    if (!empty($profiles)) {
      $entityAltered = FALSE;
      /** @var \Drupal\profile\Entity\Profile $profile */
      $profile = reset($profiles);
      $this->mapDataToFields($profile, $userData['userData'], $entityAltered);

      // Sync the user picture.
      $this->syncUserPicture($profile, $entityAltered, $userData);

      if ($entityAltered) {
        $profile->save();
      }
    }
  }

  /**
   * Sync the user picture if needed and wanted.
   *
   * @param \Drupal\user\UserInterface|\Drupal\profile\Entity\Profile $entity
   *   The user or profile entity.
   * @param bool $entityAltered
   *   If the entity is altered or not. Used later on.
   * @param array $userData
   *   The user data array containing the image.
   */
  private function syncUserPicture(&$entity, &$entityAltered, array $userData) {
    if (!empty($this->config->get('sync_profile_image'))) {
      $entityField = $this->config->get('profile_image_mapping');
      if ($entity->hasField($entityField)) {
        if ($userData && $userData['imageBase64']) {
          $data = base64_decode($userData['imageBase64']);

          // Setup the image file uri.
          $fieldDefinition = $entity->getFieldDefinition($entityField);
          if ($fieldDefinition) {
            $fieldDefinitionSettings = $fieldDefinition->getSettings();
            $uri = $fieldDefinitionSettings['uri_scheme'] . '://';
            $mimes = new MimeTypes();
            $imageName = $entity->id() . '.' . $mimes->getExtension($userData['imageMeta']['@odata.mediaContentType']);

            $imageFile = file_save_data($data, $uri . $imageName, FileSystemInterface::EXISTS_REPLACE);

            // Save the file to the entity .
            if ($imageFile) {
              $entity->set($entityField, $imageFile->id());
              $entityAltered = TRUE;
            }
          }
        }
      }
    }
  }

  /**
   * Map the user data to field in a entity.
   *
   * @param \Drupal\user\UserInterface|\Drupal\profile\Entity\Profile $entity
   *   The user or profile entity.
   * @param array $userData
   *   The user data.
   * @param bool $entityAltered
   *   If the entity is altered or not. Used later on.
   */
  private function mapDataToFields(&$entity, array $userData, &$entityAltered = FALSE) {
    $mapping = $this->getFieldMapping();

    if (!empty($mapping)) {
      // Loop through the mappings.
      foreach ($mapping as $userDataField => $entityField) {
        // Check if the fields exist in both the data and the user.
        if (isset($userData[$userDataField]) && $entity->hasField($entityField)) {
          $entity->set($entityField, $userData[$userDataField]);
          $entityAltered = TRUE;
        }
      }
    }
  }

  /**
   * Get the field mappings from the settings.
   *
   * @return array
   *   The mapping as graph_api_field => drupal_field.
   */
  private function getFieldMapping() {
    $mapping = [];
    $data = $this->config->get('graph_data_mapping');
    $lines = preg_split('/\r\n|\r|\n/', $data);

    if (!empty($lines)) {
      foreach ($lines as $line) {
        $field = explode('|', $line);
        $mapping[trim($field[0])] = trim($field[1]);
      }
    }

    return $mapping;
  }

}
